@@ -15,7 +15,9 @@ def parseUserId(obj: str):
     # TODO: add regex here (\S+):\/\/(\S+?):(\S+)
     service, user = obj.split("://")
     userId, password = user.split(":")
-    service = service.replace("port-", "")
+
+    if not "port-" in service:
+        service = "port-{}".format(service)
 
     if password == "None":
         password = None
@@ -31,7 +33,7 @@ def parseUserId(obj: str):
 
 def parseToken(token: Token):
     serviceport = "{}".format(token.service.servicename)
-    if not "port" in serviceport:
+    if not "port-" in serviceport:
         serviceport = "port-{}".format(serviceport)
 
     data = {
@@ -88,6 +90,9 @@ def loadToken(userId: str, service: Union[str, BaseService]) -> str:
     except:
         servicename = service
 
+    if not servicename.startswith("port-"):
+        servicename = "port-{}".format(servicename)
+
     result = requests.get(
         f"{tokenStorageURL}/user/{userId}/service/{servicename}",
         verify=(os.environ.get("VERIFY_SSL", "True") == "True"),
